unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, DirectX, DDUtils1;

const
  WM_RUNAPP = WM_USER;

type
  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormShow(Sender: TObject);
  private
    FDirectDraw: IDirectDraw2;           // DirectDraw2 object
    FPrimarySurface: IDirectDrawSurface; // DirectDraw primary surface
    FBackSurface: IDirectDrawSurface;    // DirectDraw back surface
    FSecondSurface: IDirectDrawSurface;
    FXAdd,FYAdd: Integer;
    FRect:TRect;
    procedure InitFail;
    procedure Drawer;
    procedure ProgIdle(Sender: TObject; var Done: Boolean);
    procedure StartDX;
    procedure GetPicture;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
implementation

{$R *.DFM}

function  DDLoadBitmap(DirectDraw      : IDirectDraw2;
                        const BitmapName: string;var  Width, Height: integer):IDirectDrawSurface;
var
  Bitmap     : HBitmap;
  BM         : Windows.TBitmap;
  SurfaceDesc: TDDSurfaceDesc;
begin
 Bitmap:= LoadImage(0, PChar(BitmapName), IMAGE_BITMAP, Width, Height,
                           LR_LOADFROMFILE or LR_CREATEDIBSECTION);
 if Bitmap = 0 then
  Raise Exception.CreateFmt('Unable to load bitmap %s', [ BitmapName ]);

 GetObject(Bitmap, SizeOf(BM), @BM);

 FillChar(SurfaceDesc, SizeOf(SurfaceDesc), 0);
 with SurfaceDesc do
  begin
   dwSize:= SizeOf(SurfaceDesc);
   dwFlags:= DDSD_CAPS or DDSD_HEIGHT or DDSD_WIDTH;
   ddsCaps.dwCaps:= DDSCAPS_OFFSCREENPLAIN;
   dwWidth:= BM.bmWidth;
   dwHeight:= BM.bmHeight;
  end;
 if DirectDraw.CreateSurface(SurfaceDesc, Result, NIL) <> DD_OK then
  Raise Exception.Create('CreateSurface failed');

 DDCopyBitmap(Result, Bitmap, 0, 0, 0, 0);
 Width:=BM.bmWidth;
 Height:=BM.bmHeight;
 if Bitmap <> 0 then DeleteObject(Bitmap);
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
 FYAdd := 2;
 FXAdd := 2;
 Application.OnIdle := ProgIdle;
end;

procedure TForm1.ProgIdle(Sender: TObject; var Done: Boolean);
var
 hr: HResult;
begin
 Done := False;
 Drawer;

 while (True) do
  begin
   hr := FPrimarySurface.Flip(nil, 0);
   if (hr = DD_OK) then break;

   if(hr = DDERR_SURFACELOST) then
    hr := FPrimarySurface.Restore();

    if(hr <> DDERR_WASSTILLDRAWING) then break;
  end;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 if(FDirectDraw <> nil) then
  begin
   FDirectDraw.FlipToGDISurface;
   FDirectDraw.SetCooperativeLevel(Handle, DDSCL_NORMAL);
   if FSecondSurface <> nil then FSecondSurface := nil;
   if FBackSurface <> nil then FBackSurface := nil;
   if FPrimarySurface <> nil then FPrimarySurface := nil;
   FDirectDraw := nil;
  end;
end;

procedure TForm1.StartDX;
var
 r: HRESULT;
 SurfaceDesc: TDDSURFACEDESC ;
 DDSCaps: TDDSCAPS;
 ADirectDraw: IDirectDraw;
begin
 r := DirectDrawCreate(nil, ADirectDraw, nil);
 if(r = DD_OK) then
  begin
   ADirectDraw.QueryInterface(IID_IDirectDraw2, FDirectDraw);
   ADirectDraw := nil;
   r := FDirectDraw.SetCooperativeLevel(Handle,
      DDSCL_EXCLUSIVE or DDSCL_FULLSCREEN);
   if(r <> DD_OK) then close;
   r := FDirectDraw.SetDisplayMode(800, 600, 8, 0,0 );
   if(r <> DD_OK) then close;

   SurfaceDesc.dwSize := sizeof(SurfaceDesc);
   SurfaceDesc.dwFlags := DDSD_CAPS or DDSD_BACKBUFFERCOUNT;
   SurfaceDesc.ddsCaps.dwCaps := DDSCAPS_PRIMARYSURFACE or
                          DDSCAPS_FLIP or DDSCAPS_COMPLEX;
   SurfaceDesc.dwBackBufferCount := 1;
   r := FDirectDraw.CreateSurface(SurfaceDesc, FPrimarySurface, nil);
   if(r <> DD_OK) then close;

   ddscaps.dwCaps := DDSCAPS_BACKBUFFER;
   r := FPrimarySurface.GetAttachedSurface(ddscaps,FBackSurface);
   if(r <> DD_OK) then close;
   GetPicture;
  end
 else close;
end;

procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 case Key of
  VK_ESCAPE:
   begin
    Close;
   end;
 end;  
end;

procedure TForm1.Drawer;
begin
 FRect.left := FRect.Left + FXAdd;
 FRect.Top := FRect.Top + FYAdd;

 if (FRect.left+50 > 795) then FXAdd := -2;
 if (FRect.left < 3) then FXAdd := 2;
 if (FRect.Top+50 > 595) then FYAdd := -2;
 if (FRect.Top < 3) then FYAdd := 2;

 FBackSurface.BltFast(FRect.Left, FRect.Top, FSecondSurface, Rect(0,0,22,22), DDBLTFAST_NOCOLORKEY );
end;

procedure TForm1.FormShow(Sender: TObject);
begin
 StartDX();
end;

procedure TForm1.GetPicture;
begin
 FSecondSurface := DDLoadBitmap(FDirectDraw, '1.bmp', FRect.Left, FRect.Top);
 if(FSecondSurface = nil) then
  begin
   InitFail();
   Exit;
  end;
end;

procedure TForm1.InitFail;
begin
 if FDirectDraw <> nil then
  begin
   FDirectDraw.FlipToGDISurface;
   MessageBox(Handle, 'DirectDraw Init FAILED', '', MB_OK );
   Close();
  end;
end;

end.
